@@ -776,7 +776,7 @@ sub detect_relocation {
     return unless -e $config_file;
 
     my %inc = %INC;
-    eval { require "$config_file" };
+    eval { require "./$config_file" };
     %INC = %inc; # be stealth
     warn($@), return if $@;
 
@@ -1097,9 +1097,6 @@ sub generate_script {
 
     my %args = @Apache::TestMM::Argv;
     while (my($k, $v) = each %args) {
-        unless (defined $v) {
-            die "key $k has no value";
-        }
         $v =~ s/\|/\\|/g;
         $body .= "\n\$Apache::TestConfig::Argv{'$k'} = q|$v|;\n";
     }
