@@ -23,6 +23,7 @@ use DirHandle ();
 use File::Path ();
 use File::Copy 'cp';
 use File::Basename;
+use File::Spec::Functions qw(devnull);
 use Apache::TestConfig ();
 use Apache::TestTrace;
 
@@ -46,17 +47,45 @@ my $cakey    = 'keys/ca.pem';
 my $cacert   = 'certs/ca.crt';
 my $capolicy = '-policy policy_anything';
 my $cacrl    = 'crl/ca-bundle.crl';
+my $dgst     = 'sha256';
 
 #we use the same password for everything
 my $pass    = 'httpd';
 my $passin  = "-passin pass:$pass";
 my $passout = "-passout pass:$pass";
 
+# (limited) subjectAltName otherName testing
+my $san_msupn  = ', otherName:msUPN;UTF8:$mail';
+my $san_dnssrv = ', otherName:1.3.6.1.5.5.7.8.7;IA5:_https.$CN';
+
 # in 0.9.7 s/Email/emailAddress/ in DN
 my $email_field = Apache::Test::normalize_vstring($version) <
                   Apache::Test::normalize_vstring("0.9.7") ?
                   "Email" : "emailAddress";
 
+# downgrade to SHA-1 for OpenSSL before 0.9.8
+if (Apache::Test::normalize_vstring($version) <
+    Apache::Test::normalize_vstring("0.9.8")) {
+    $dgst = 'sha1';
+    # otherNames in x509v3_config are not supported either
+    $san_msupn = $san_dnssrv = "";
+}
+
+my $sslproto = "all";
+
+eval { require Net::SSLeay; };
+if (Apache::Test::normalize_vstring($version) >= 
+    Apache::Test::normalize_vstring("1.1.1")
+    && !defined(&Net::SSLeay::CTX_set_post_handshake_auth)) {
+    # OpenSSL 1.1.1 disables PHA by default client-side in TLSv1.3 but
+    # most clients are not updated to enable it (at time of writing).
+    # Many mod_ssl tests require working PHA, so disable v1.3 unless
+    # using an updated Net::SSLeay. This is strictly insufficient
+    # since an updated IO::Socket::SSL is also needed; to be
+    # continued.  Ref: https://github.com/openssl/openssl/issues/6933
+    $sslproto = "all -TLSv1.3";
+}
+
 my $ca_dn = {
     asf => {
         C  => 'US',
@@ -79,6 +108,9 @@ my $cert_dn = {
     },
     client_ok => {
     },
+    client_colon => {
+        CN => "user:colon",
+    },
     client_revoked => {
     },
     server => {
@@ -100,8 +132,9 @@ my $cert_dn = {
 };
 
 #generate DSA versions of the server certs/keys
-while (my($key, $val) = each %$cert_dn) {
+for my $key (keys %$cert_dn) {
     next unless $key =~ /^server/;
+    my $val = $$cert_dn{$key};
     my $name = join '_', $key, 'dsa';
     $cert_dn->{$name} = { %$val }; #copy
     $cert_dn->{$name}->{OU} =~ s/rsa/dsa/;
@@ -201,16 +234,19 @@ sub config_file {
     return $file if -e $file;
 
     my $dn = dn($name);
-    my $db = sslca_db($name);
+    my $db = SSLCA_DB;
 
-    writefile($db, '', 1);
+    writefile($db, '', 1) unless -e $db;
 
     writefile($file, <<EOF);
+mail                   = $dn->{$email_field}
+CN                     = $dn->{CN}
+
 [ req ]
 distinguished_name     = req_distinguished_name
 attributes             = req_attributes
 prompt                 = no
-default_bits           = 1024
+default_bits           = 2048
 output_password        = $pass
 
 [ req_distinguished_name ]
@@ -219,8 +255,8 @@ ST                     = $dn->{ST}
 L                      = $dn->{L}
 O                      = $dn->{O}
 OU                     = $dn->{OU}
-CN                     = $dn->{CN}
-emailAddress           = $dn->{$email_field}
+CN                     = \$CN
+$email_field           = \$mail
 
 [ req_attributes ]
 challengePassword      = $pass
@@ -241,7 +277,7 @@ private_key      = $cakey       # The private key
 
 default_days     = 365          # how long to certify for
 default_crl_days = 365          # how long before next CRL
-default_md       = md5          # which md to use.
+default_md       = $dgst        # which md to use.
 preserve         = no           # keep passed DN ordering
 
 [ policy_anything ]
@@ -251,12 +287,15 @@ localityName            = optional
 organizationName        = optional
 organizationalUnitName  = optional
 commonName              = supplied
-emailAddress            = optional
+$email_field            = optional
 
-[ comment ]
+[ client_ok_ext ]
 nsComment = This Is A Comment
 1.3.6.1.4.1.18060.12.0 = DER:0c064c656d6f6e73
+subjectAltName = email:\$mail$san_msupn
 
+[ server_ext ]
+subjectAltName = DNS:\$CN$san_dnssrv
 EOF
 
     return $file;
@@ -304,12 +343,12 @@ sub new_key {
         #this takes a long time so just do it once
         #don't do this in real life
         unless (-e 'dsa-param') {
-            openssl dsaparam => '-inform PEM -out dsa-param 1024';
+            openssl dsaparam => '-inform PEM -out dsa-param 2048';
         }
-        openssl gendsa => "dsa-param $out";
+        openssl gendsa => "$out dsa-param";
     }
     else {
-        openssl genrsa => "$out 1024";
+        openssl genrsa => "$out 2048";
     }
 }
 
@@ -328,7 +367,9 @@ sub sign_cert {
     my $name = shift;
     my $exts = '';
 
-    $exts = ' -extensions comment' if $name =~ /client_ok/;
+    $exts = ' -extensions client_ok_ext' if $name =~ /client_ok/;
+
+    $exts = ' -extensions server_ext' if $name =~ /server/;
 
     openssl ca => "$capolicy -in csr/$name.csr -out certs/$name.crt",
                   $passin, config($name), '-batch', $exts;
@@ -344,11 +385,6 @@ sub export_cert {
                       "-out export/$name.p12", $passin, $passout;
 }
 
-sub sslca_db {
-    my $name = shift;
-    return "$name-" . SSLCA_DB;
-}
-
 sub revoke_cert {
     my $name = shift;
 
@@ -357,15 +393,6 @@ sub revoke_cert {
     #revokes in the SSLCA_DB database
     openssl ca => "-revoke certs/$name.crt", @args;
 
-    my $db = sslca_db($name);
-    unless (-e $db) {
-        #hack required for win32
-        my $new = join '.', $db, 'new';
-        if (-e $new) {
-            cp $new, $db;
-        }
-    }
-
     #generates crl from the index.txt database
     openssl ca => "-gencrl -out $cacrl", @args;
 }
@@ -546,9 +573,22 @@ sub gendir {
 }
 
 sub version {
-    my $version = qx($openssl version);
-    return $1 if $version =~ /^OpenSSL (\S+) /;
-    return 0;
+    my $devnull = devnull();
+    my $version = qx($openssl version 2>$devnull);
+    return $1 if $version =~ /^\S+SSL (\S+)/;
+    die "FATAL: unable to determine openssl version via `$openssl version` from: $version";
+}
+
+sub dgst {
+    return $dgst;
+}
+
+sub email_field {
+    return $email_field;
+}
+
+sub sslproto {
+    return $sslproto;
 }
 
 1;
